/*
* Copyright (C) 2020 by Savoir-faire Linux
* Authors: William Enright <william.enright@savoirfairelinux.com>
*          Ndeye Anna Ndiaye <anna.ndiaye@savoirfairelinux.com>
*          Johnny Flores <johnny.flores@savoirfairelinux.com>
*          Mohammed Raza <mohammed.raza@savoirfairelinux.com>
*          Felix Sidokhine <felix.sidokhine@savoirfairelinux.com>
*
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/
package net.jami.datastore.dao;

import lombok.extern.slf4j.Slf4j;
import net.jami.datastore.main.DataStore;
import net.jami.jams.common.dao.StatementList;
import net.jami.jams.common.dao.connectivity.SQLConnection;
import net.jami.jams.common.objects.user.User;

import java.sql.PreparedStatement;
import java.sql.SQLException;

@Slf4j
public class UserDao extends AbstractDao<User> {

    public UserDao() {
        SQLConnection connection = DataStore.connectionPool.getConnection();
        try {
            this.setTableName("users");
            this.setTClass(User.class);
            String createTable = "CREATE TABLE users (" +
                    "username varchar(255), " +
                    "password varchar(255)," +
                    "userType varchar(10)," +
                    "realm varchar(255)," +
                    "ethAddress varchar(255)," +
                    "ethKey varchar(255)," +
                    "jamiId varchar(255)," +
                    "certificate varchar(5000), "+
                    "privatekey varchar(5000)," +
                    "accessLevel varchar(10),"+
                    "needsPasswordReset varchar(10),"+
                    "PRIMARY KEY (username))";
            PreparedStatement ps = connection.getConnection().prepareStatement(createTable);
            ps.execute();
        }
        catch (SQLException e){
            log.error("Could not create the device table with error " + e.getMessage());
        }
        finally {
            DataStore.connectionPool.returnConnection(connection);
        }
    }

    @Override
    public boolean storeObject(User object) {
        SQLConnection connection = DataStore.connectionPool.getConnection();
        try{
            PreparedStatement ps = connection.getConnection().prepareStatement("INSERT INTO users " +
                    "(username, password, userType, realm, ethAddress, ethKey, jamiId,certificate, privatekey, accessLevel," +
                    "needsPasswordReset) " +
                    "VALUES " +
                    "(?, ?, ?, ?, ?, ?, ?,?, ?,?, ?)");
            ps = object.getInsert(ps);
            return ps.executeUpdate() != 0;
        }
        catch (Exception e){
            log.error("An error has occurred while trying to store a user: " + e.toString());
            return false;
        }
        finally {
            DataStore.connectionPool.returnConnection(connection);
        }
    }

    @Override
    public boolean updateObject(StatementList update, StatementList constraints) {

        String pw = update.getStatements().get(0).getValue();
        String user = update.getStatements().get(1).getValue();
        String needsPasswordReset = "";
        if (update.getStatements().size() >= 3 && update.getStatements().get(2) != null)
            needsPasswordReset = update.getStatements().get(2).getValue();

        SQLConnection connection = DataStore.connectionPool.getConnection();

        try{
            PreparedStatement ps = connection.getConnection().prepareStatement("UPDATE users SET password = ? WHERE username = ?");
            ps.setString(1, pw);
            ps.setString(2, user);
            ps.executeUpdate();

            ps = connection.getConnection().prepareStatement("UPDATE users SET needsPasswordReset = ? WHERE username = ?");
            if (!needsPasswordReset.isEmpty())
                ps.setString(1, needsPasswordReset);
            else
                ps.setString(1, "false");

            ps.setString(2, user);
            return ps.executeUpdate() != 0;
        }
        catch (Exception e){
            log.error("An error has occurred while trying to update a user: " + e.toString());
            return false;
        }
        finally {
            DataStore.connectionPool.returnConnection(connection);
        }
    }
}
