/*
* Copyright (C) 2020 by Savoir-faire Linux
* Authors: William Enright <william.enright@savoirfairelinux.com>
*          Ndeye Anna Ndiaye <anna.ndiaye@savoirfairelinux.com>
*          Johnny Flores <johnny.flores@savoirfairelinux.com>
*          Mohammed Raza <mohammed.raza@savoirfairelinux.com>
*          Felix Sidokhine <felix.sidokhine@savoirfairelinux.com>
*
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/
package launcher.loaders;

import lombok.extern.slf4j.Slf4j;
import net.jami.jams.common.updater.AppUpdater;
import net.jami.jams.common.utils.LibraryLoader;
import net.jami.jams.server.Server;

@Slf4j
public class ServerLoader {

    public static Server loadServer(AppUpdater appUpdater, String port, String serverCertificate, String serverPrivateKey) {
        try {
            Class<?> cls = LibraryLoader.classLoader.loadClass("net.jami.jams.server.Server");
            log.info("Server found, starting......");
            return  (Server) cls.getConstructor(AppUpdater.class, String.class, String.class, String.class).newInstance
                    (appUpdater, port, serverCertificate, serverPrivateKey);
        }
        catch (Exception e){
            log.error("Could not find and start the server with error {}",e.getCause().getClass());
            return null;
        }
    }
}
